class GoogleColabSeleniumError(Exception):
    """Base Google Colab Selenium Exception"""

    report_msg = """\n\t
        If you believe this isn't due to a configuration error on your end, please
        report the issue with any details you can provide about the steps leading
        up to this error.\n

        You can submit your report here:
        https://github.com/jpjacobpadilla/Google-Colab-Selenium/issues/new

        We apologize for any inconvenience!
    """

    default_message = "An error occurred while attempting to initialize/setup Selenium."

    def __init__(self, message = None):
        super().__init__(
            f'{message or self.default_message}{self.report_msg}'
        )


class InstallChromeError(GoogleColabSeleniumError):
    """Exception raised when installation of Chrome fails."""
    pass

class ChromeDriverPathError(GoogleColabSeleniumError):
    """Exception raised when ChromeDriver path cannot be found or is incorrect."""
    pass

class StartingChromeDriverError(GoogleColabSeleniumError):
    """Exception raised when ChromeDriver fails to start."""
    pass


import subprocess
import shutil

from google_colab_selenium.spinner import Spinner
from google_colab_selenium.exceptions import (
    ChromeDriverPathError, InstallChromeError,
)

from selenium.webdriver.chrome.service import Service
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.selenium_manager import SeleniumManager


class ColabSeleniumManager:
    default_colab_options = [
        '--headless',
        '--no-sandbox',
        '--disable-dev-shm-usage',
        '--lang=en'
    ]

    _downloaded_chrome = False

    download_command = ['curl', '-o', 'google-chrome-stable_current_amd64.deb', 'https://dl.google.com/linux/direct/google-chrome-stable_current_amd64.deb']
    install_command = ['sudo', 'apt', 'install', './google-chrome-stable_current_amd64.deb', '-y']
    clean_up_command = ['rm', 'google-chrome-stable_current_amd64.deb']

    chromedriver_path: str = None

    def __init__(self, base_options: Options):
        if not self._downloaded_chrome:
            # Checks if Chrome was already installed. The class may of been reset.
            if shutil.which("google-chrome-stable") is None:
                self.install_chrome()
            else:
                self._downloaded_chrome = True

        self.options = self.default_options(base_options or Options())
        self.service = self.get_service()

    @classmethod
    def install_chrome(cls, *args, **kwargs) -> None:
        """
        To Install Google-Chrome-Stable, the first command uses CURL to download
        the debian file. Next Advanced Package Tool installs the file and once
        it's installed, the .deb file, which is no longer needed, is deleted.
        """
        try:
            with Spinner('Downloading Google Chrome', done='Downloaded Google Chrome'):
                subprocess.run(cls.download_command, check=True)
                subprocess.run(cls.install_command, check=True)
                subprocess.run(cls.clean_up_command, check=True)

        except Exception as e:
            raise InstallChromeError("Failed to install Google Chrome.") from e

        else:
            cls._downloaded_chrome = True

    @classmethod
    def default_options(cls, options: Options) -> Options:
        for default in cls.default_colab_options:
            options.add_argument(default)

        return options

    @classmethod
    def get_service(cls) -> Service:
        path = cls.chromedriver_path or cls.prepare_driver()
        return Service(path)

    @classmethod
    def prepare_driver(cls) -> str:
        try:
            path = SeleniumManager().driver_location(Options())
            cls.chromedriver_path = path
            return path

        except Exception as e:
            raise ChromeDriverPathError("Failed to find ChromeDriver.") from e








from google_colab_selenium.exceptions import (
    GoogleColabSeleniumError,
    InstallChromeError,
    ChromeDriverPathError,
    StartingChromeDriverError
)

__all__ = (
    'ChromeDriver', 
    'UndetectedChromeDriver', 
    'Chrome', 
    'UndetectedChrome',
    'GoogleColabSeleniumError',
    'InstallChromeError',
    'ChromeDriverPathError',
    'StartingChromeDriverError'
)


import os
assert 'COLAB_GPU' in os.environ, "This package should only be used in Google Colab Notebooks."


def __getattr__(name):
    if name == 'ChromeDriver' or name == 'Chrome':
        from google_colab_selenium.chromedriver import ChromeDriver
        return ChromeDriver

    if name == 'UndetectedChromeDriver' or name == 'UndetectedChrome':
        from google_colab_selenium.undetected_chromedriver import UndetectedChromeDriver
        return UndetectedChromeDriver

    raise AttributeError(f"module 'google_colab_selenium' has no attribute '{name}'")



from IPython.display import display, HTML, Javascript
import threading
import uuid


class Spinner:
    # Initializer accepts a message
    def __init__(self, message: str, done: str):
        self.message = message
        self.done_message = done
        self.stop_event = threading.Event()

    def __enter__(self):
        self.show_spinner(self.message)
        return self

    def __exit__(self, *args, **kwargs):
        self.remove_spinner()

    def show_spinner(self, text):
        self.spinner_id = uuid.uuid4()

        spinner_html = f"""
            <div class="spinner-container">
                <div class="spinner" id="{self.spinner_id}-circle"></div>
                <div class="spinner-text" id="{self.spinner_id}-text">{text}</div>
            </div>
            <style>
                @keyframes spin {{
                    from {{ transform: rotate(0deg); }}
                    to {{ transform: rotate(360deg); }}
                }}

                .spinner-container {{
                    display: flex;
                    align-items: center;
                    margin-bottom: 3px;
                }}

                .spinner {{
                    border: 3px solid rgba(0, 0, 0, 0.1);
                    border-left-color: lightblue;
                    border-radius: 50%;
                    width: 12px;
                    height: 12px;
                    animation: spin 1s linear infinite;
                }}

                .spinner-text {{
                    padding-left: 6px;
                }}
            </style>
        """
        display(HTML(spinner_html))

    def remove_spinner(self):
        js_code = f"""
            const element = document.getElementById("{self.spinner_id}-circle");
            element.style.border = "3px solid limegreen";
            element.style.animation = "none";

            const text = document.getElementById("{self.spinner_id}-text");
            text.innerText = "{self.done_message}";
        """
        display(Javascript(js_code))



from google_colab_selenium.colab_selenium_manager import ColabSeleniumManager
from google_colab_selenium.spinner import Spinner
from google_colab_selenium.exceptions import StartingChromeDriverError

from selenium.webdriver.chrome.options import Options

try:
    import undetected_chromedriver as uc
except ImportError as e:
    raise ImportError('''
        Please install google-colab-selenium with the "undetected" 
        extra -> pip3 install google-colab-selenium[undetected]
    ''')

    
class UndetectedChromeDriver(uc.Chrome):
    """
    Instead of using ChromeDriver, which is easy to detect, you can use undetected-chromedriver.

    https://github.com/ultrafunkamsterdam/undetected-chromedriver

    This package is a great start to making Selenium undetectable,
    but you still need to act like a human.

    The ColabSeleniumManager class installs Google-Chrome-Stable and adds the
    nessasary headers to use in a Colab Notebook.

    The headers that are automatically added are:
        --headless
        --no-sandbox
        --disable-dev-shm-usage
        --lang=en
    """
    def __init__(self, options: Options = None, keep_alive: bool = True):

        self.manager = ColabSeleniumManager(options or uc.ChromeOptions())

        try:
            with Spinner('Initializing Chromedriver', done='Initialized Chromedriver'):
                super().__init__(
                    service=self.manager.service,
                    options=self.manager.options,
                    keep_alive=keep_alive
                )

        except Exception as e:
            raise StartingChromeDriverError("""
                Failed to start ChromeDriver. This could be due to a number
                of factors, such as missing dependencies or incorrect
                configuration settings.
            """) from e


from setuptools import setup


with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='google-colab-selenium',
    description='Easily use Selenium in Google Colab Notebooks!',
    version='1.0.7',
    packages=['google_colab_selenium'],
    install_requires=['selenium'],
    extras_require={
        'undetected': ['undetected-chromedriver']
    },
    author = 'Jacob Padilla',
    author_email = 'jp@jacobpadilla.com',
    url='https://github.com/jpjacobpadilla/Google-Colab-Selenium',
    license='MIT',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='selenium google-colab webdriver automation chromedriver'
)
